function [output,DICfilename, measurementmatrix, DICbw, allcells]=cellID(DICimage)
%http://www.mathworks.com/products/image/examples.html?file=/products/demos/shipping/images/ipexcel

DICfilename = DICimage; 

%Step 1: Read Image

%highthroughput_cellID use
I = imread(DICimage); % convert image from rgb->gray for use with edge function

%highthroughput_imagegenerator use
I =  I(:,:,1);

%figure, subplot(2,4,1), imshow(I), title('original image');

%Step 1.5: Smooth Image
%hsize = [3 3];
hsize = [5 5];
h = fspecial('average', hsize); %http://www.mathworks.com/help/toolbox/images/ref/fspecial.html
I = imfilter(I,h,'replicate'); 
I = imfilter(I,h,'replicate'); 

%Step 2: Detect Entire Cell
[junk threshold] = edge(I, 'sobel');
fudgeFactor = .5;
BWs = edge(I,'sobel', threshold * fudgeFactor);
%subplot(2,4,2), imshow(BWs), title('binary gradient mask');

%Step 3: Dilate the Image
se90 = strel('line', 3, 90);
se0 = strel('line', 3, 0);
BWsdil = imdilate(BWs, [se90 se0]);
%subplot(2,4,3), imshow(BWsdil), title('dilated gradient mask');

%Step 4: Fill Interior Gaps
BWdfill = imfill(BWsdil, 'holes');
%subplot(2,4,4), imshow(BWdfill), title('binary image with filled holes');

%Step 5: Remove Connected Objects on Border
%BWnobord = imclearborder(BWdfill, 4);
%subplot(2,4,5), imshow(BWnobord), title('cleared border image');

%Step 6: Smoothen the Object
seD = strel('diamond',1);
%BWfinal = imerode(BWnobord,seD); %Not ignoring step 5
BWfinal = imerode(BWdfill,seD); %Ignoring step 5
BWfinal = imerode(BWfinal,seD);
%subplot(2,4,6), imshow(BWfinal), title('segmented image');

originalImage=I;
%figure, subplot(2,2,1), imshow(originalImage), title('Original Image')

binaryImage=BWfinal;
%subplot(2,2,2), figure, imshow(binaryImage), title('Starting binary image')

labeledImage = bwlabel(binaryImage, 8);     % Label each blob so we can make measurements of it. Can also use bwconncomp
coloredLabels = label2rgb (labeledImage, 'hsv', 'k', 'shuffle'); % pseudo random color labels: label2rgb picks colors from entire range of available colors
%subplot(2, 2, 3), imagesc(coloredLabels), axis square
%caption = sprintf('Pseudo colored labels, from label2rgb().\nBlobs are numbered from top to bottom, then from left to right.');
%title(caption);

% Get all the blob properties. 
blobMeasurements = regionprops(labeledImage, binaryImage, 'all'); %STATS = regionprops(..., I, properties) measures a set of properties for each labeled region in the image I. The first input to regionpropseither BW, CC, or Lidentifies the regions in I. The sizes must match: size(I) must equal size(BW), CC.ImageSize, or size(L).
numberOfBlobs = size(blobMeasurements, 1);

%{
output=[];
for x=1:numberOfBlobs
    output(x,:)=[blobMeasurements(x).Area, blobMeasurements(x).MajorAxisLength, blobMeasurements(x).MinorAxisLength, blobMeasurements(x).Eccentricity, blobMeasurements(x).EquivDiameter, blobMeasurements(x).Perimeter];
end %for
%}

%Call subfunction selectcells to obtain cells and erase background
[output, measurementmatrix, DICbw, allcells] = selectcells(blobMeasurements,labeledImage,originalImage,binaryImage,I);

    
end %function cellID


function [output, measurementmatrix, DICbw, allcells]=selectcells(blobMeasurements,labeledImage,originalImage,binaryImage,DIC)

% Select certain blobs based on measurement criteria using the ismember function.
    allBlobAreas = [blobMeasurements.Area];
    allBlobEquivDiam= [blobMeasurements.EquivDiameter];
    allBlobPerimeter= [blobMeasurements.Perimeter];
    allBlobEccentricities = [blobMeasurements.Eccentricity];
% Get a list of the blobs that meet our criteria and we need to keep. 
    allowablePerimeterIndexes = allBlobPerimeter > 100; 
    %allowableEccentricityIndexes =  (allBlobEccentricities < 0.70);
    allowableAreaIndexes = allBlobAreas > 1500;
% keeperIndexes = find(allowableIntensityIndexes & allowableAreaIndexes); 
    %keeperIndexes = find(allowablePerimeterIndexes & allowableEccentricityIndexes & allowableAreaIndexes);
    keeperIndexes = find(allowablePerimeterIndexes & allowableAreaIndexes);

% Use ismember: Extract only those blobs that meet our criteria, and eliminate those blobs that don't meet our criteria. 
    keeperBlobsImage = ismember(labeledImage, keeperIndexes); 
% Re-label with only the keeper blobs kept.
    labeledCellImage = bwlabel(keeperBlobsImage, 8);     % Label each blob so we can make measurements of it

figure, subplot(1,2,1), imshow(originalImage), title('Original Image')

% Take new measurements of cells
newblobMeasurements = regionprops(labeledCellImage, binaryImage, 'all'); %STATS = regionprops(..., I, properties) measures a set of properties for each labeled region in the image I. The first input to regionpropseither BW, CC, or Lidentifies the regions in I. The sizes must match: size(I) must equal size(BW), CC.ImageSize, or size(L).
newnumberOfBlobs = size(newblobMeasurements, 1);


for i = 1:newnumberOfBlobs
    measurementmatrix(i,1) = newblobMeasurements(i).Eccentricity;
    measurementmatrix(i,2) = newblobMeasurements(i).Area;
end %for

% Now use the keeper blob as a mask on the original image.
% This will let us display the original image in the regions of the keeper blobs.
    maskedImageCell = originalImage; % Simply a copy at first.
    %maskedImageDime = rgb2gray(originalImage); %If you don't do this the only R channel of RGB will be set to 0
    maskedImageCell(~keeperBlobsImage) = 0;  % Set all non-keeper pixels to zero.
    
    subplot(1, 2, 2); 
    imshow(maskedImageCell);
    %axis square;
    title('Objects that meet criteria from the original image');

    
% Loop through and number each cell
fontSize = 10;	% Used to control size of "blob number" labels put atop the image.
labelShiftX = 0;	% Used to align the labels in the centers of the objects.
blobECD = zeros(1, newnumberOfBlobs);

for k = 1 : newnumberOfBlobs % Loop through all blobs.
	blobCentroid = newblobMeasurements(k).Centroid; % Get centroid.
	text(blobCentroid(1) + labelShiftX, blobCentroid(2), num2str(k), 'FontSize', fontSize, 'FontWeight', 'Bold','Color','r');
end %for

%figure; imshow(im2bw(maskedImageCell,0))  
allcells = maskedImageCell;
DICbw = im2bw(maskedImageCell,0);
%call function selectsinglecell to create images of each single cell
[output] = selectsinglecell(labeledCellImage,newnumberOfBlobs,DIC);

end % function selectcells



function [IndivCellImages]=selectsinglecell(labeledCellImage,newnumberOfBlobs,DIC)

for i = 1:newnumberOfBlobs
    IndivCellImages(i).image = (labeledCellImage == i); %field image refers to mask image
    temporaryDIC = DIC;
    A = (labeledCellImage == i);
    temporaryDIC(~A) = 0;
    IndivCellImages(i).DICimage = temporaryDIC; %field DICimage refers to actual DIC image of cells
end %for

%{
figure, %show each individual cell mask

for i = 1:newnumberOfBlobs
    subplot(3,3,i)
    imshow(IndivCellImages(i).image)
    title(['Cell ',num2str(i), ' Mask'])
end %for
%}
figure, %show each individual cell DIC image
for i = 1:newnumberOfBlobs
    subplot(4,4,i)
    imshow(IndivCellImages(i).DICimage)
    title(['Cell ',num2str(i), ' DIC'])
end %for


end %function selectsinglecell


